<?php
/* --------------------------------------------------------------
   ProductImportRunner.php 2022-11-09
   Gambio GmbH
   http://www.gambio.de
   Copyright (c) 2022 Gambio GmbH
   Released under the GNU General Public License (Version 2)
   [http://www.gnu.org/licenses/gpl-2.0.html]
   --------------------------------------------------------------
*/
declare(strict_types=1);

namespace GXModules\Gambio\Afterbuy\Admin\Classes\Products;

use GambioAfterbuyConfigurationStorage;
use GXModules\Gambio\Afterbuy\Admin\Classes\AfterbuyXML\AfterbuyXMLService;
use GXModules\Gambio\Afterbuy\Admin\Classes\Catalogs\AfterbuyCatalogRepository;
use GXModules\Gambio\Afterbuy\Admin\Classes\Products\Exceptions\UnsupportedProductTypeException;
use GXModules\Gambio\Afterbuy\Admin\Classes\Products\ValueObjects\AfterbuyProduct;
use Psr\Log\LoggerAwareInterface;
use Psr\Log\LoggerInterface;
use Psr\Log\NullLogger;

class ProductImportRunner implements LoggerAwareInterface
{
    private LoggerInterface                    $logger;
    private GambioAfterbuyConfigurationStorage $configurationStorage;
    private AfterbuyXMLService                 $XMLService;
    private AfterbuyCatalogRepository          $catalogRepository;
    private AfterbuyProductImporter            $importer;
    
    
    public function __construct(
        GambioAfterbuyConfigurationStorage $configurationStorage,
        AfterbuyXMLService                 $XMLService,
        AfterbuyCatalogRepository          $catalogRepository,
        AfterbuyProductImporter            $importer
    ) {
        $this->logger               = new NullLogger();
        $this->configurationStorage = $configurationStorage;
        $this->XMLService           = $XMLService;
        $this->catalogRepository    = $catalogRepository;
        $this->importer             = $importer;
    }
    
    
    public function run(): void
    {
        $defaultCategoryId     = (int)$this->configurationStorage->get('import_products_default_category');
        $sinceTimestamp        = (int)$this->configurationStorage->get('import_since_timestamp');
        $sinceDateTime         = new \DateTimeImmutable("@{$sinceTimestamp}");
        $lastProductId         = (int)$this->configurationStorage->get('import_last_product_id');
        $productLevelThreshold = (int)$this->configurationStorage->get('import_products_level_threshold');
        $dealerGroupId         = (int)$this->configurationStorage->get('import_products_dealer_group');
        $maxProductsPerRun     = (int)$this->configurationStorage->get('import_max_products_per_run');
        
        $this->XMLService->setLogger($this->logger);
        $this->importer->setLogger($this->logger);
        
        $this->logger->info("Starting product import run");
        
        $getProductsResult = $this->XMLService->getShopProductsModifiedSinceRange($sinceDateTime,
                                                                                  $maxProductsPerRun,
                                                                                  $lastProductId);
        
        /** @var AfterbuyProduct $importAfterbuyProduct */
        foreach ($getProductsResult->getProducts() as $importAfterbuyProduct) {
            $this->logger->debug("Importing {$importAfterbuyProduct->getProductID()}");
            //$productCategoryIds = $this->catalogRepository->getCategoryIdsForAfterbuyProductId($importAfterbuyProduct->getProductID());
            $productCategoryIds = [];
            foreach ($importAfterbuyProduct->getCatalogs() as $catalogId) {
                $categoryId = $this->catalogRepository->getCategoryIdByCatalogId($catalogId);
                if ($categoryId !== null) {
                    $productCategoryIds[] = $categoryId;
                }
            }
            $this->logger->debug("Product {$importAfterbuyProduct->getProductID()}, Categories " . implode(", ",
                                                                                                           $productCategoryIds));
            if ($defaultCategoryId < 0) {
                $this->logger->info("Product {$importAfterbuyProduct->getProductID()} has no associated catalog, no default category set => not importing");
                continue;
            }
            $importCategoryIds = empty($productCategoryIds) ? [$defaultCategoryId] : $productCategoryIds;
            try {
                $gambioProductId = $this->importer->importProduct($importAfterbuyProduct,
                                                                  $importCategoryIds,
                                                                  $productLevelThreshold,
                                                                  $dealerGroupId);
                $this->logger->debug("Imported Afterbuy product {$importAfterbuyProduct->getProductID()} as {$gambioProductId} into categories "
                                    . implode(', ', $importCategoryIds));
            } catch (UnsupportedProductTypeException $upe) {
                $this->logger->debug("Product has BaseProductFlag {$importAfterbuyProduct->getBaseProductFlag()}, skipping");
                continue;
            }
        }
        
        if ($getProductsResult->isHasMoreProducts()) {
            $this->configurationStorage->set('import_last_product_id', $getProductsResult->getLastProductId());
        } else {
            $this->configurationStorage->set('import_last_product_id', 0);
            $this->configurationStorage->set('import_since_timestamp', time());
        }
    }
    
    
    public function setLogger(LoggerInterface $logger)
    {
        $this->logger = $logger;
    }
}
