<?php
/* --------------------------------------------------------------
   ProductsQuantityUpdateService.php 2022-11-25
   Gambio GmbH
   http://www.gambio.de
   Copyright (c) 2022 Gambio GmbH
   Released under the GNU General Public License (Version 2)
   [http://www.gnu.org/licenses/gpl-2.0.html]
   --------------------------------------------------------------
*/
declare(strict_types=1);

namespace GXModules\Gambio\Afterbuy\Admin\Classes;

use Doctrine\DBAL\Connection;
use GXModules\Gambio\Afterbuy\Admin\Classes\Products\ProductsMapper;
use GXModules\Gambio\Afterbuy\Admin\Classes\Products\ProductsMappingRepository;
use GXModules\Gambio\Afterbuy\Admin\Classes\Products\ValueObjects\AfterbuyProduct;
use GXModules\Gambio\Afterbuy\Admin\Classes\Products\ValueObjects\MappingMode;
use GXModules\Gambio\Afterbuy\Admin\Classes\Products\ValueObjects\ProductMapping;
use Psr\Log\LoggerAwareInterface;
use Psr\Log\LoggerInterface;

require_once(DIR_FS_CATALOG . 'gm/inc/set_shipping_status.php');

/**
 * Updates product quantities.
 *
 * This class does not use ProductReadService and ProductWriteService due to performance issues.
 */
class ProductsQuantityUpdateService implements LoggerAwareInterface
{
    protected \CI_DB_query_builder    $db;
    private LoggerInterface           $logger;
    private ProductsMappingRepository $productsMappingRepository;
    
    
    public function __construct(\CI_DB_query_builder $db)
    {
        $this->db                        = $db;
        $dbConnection                    = \LegacyDependencyContainer::getInstance()->get(Connection::class);
        $this->productsMappingRepository = new ProductsMappingRepository($dbConnection);
    }
    
    
    /**
     * Uses a set of AfterbuyProducts to update local product quantities.
     *
     * Uses the EAN (External Article Number) from Afterbuy to identify local products by their products_id.
     *
     * @param AfterbuyProduct ...$products
     *
     * @return void
     */
    public function updateProductQuantities(string $mode = 'pid_ean', AfterbuyProduct ...$products): void
    {
        $this->log("Match mode: {$mode}");
        $mapper = new ProductsMapper(new MappingMode($mode));
        $mapper->setLogger($this->logger);
        foreach ($products as $product) {
            $this->log("Afterbuy Produkt {$product->getProductID()} / {$product->getName()} / Anr {$product->getAnr()} / EXAN {$product->getEAN()}");
            
            $productMapping = $this->productsMappingRepository->findMappingByAfterbuyProduct($product);
            if ($productMapping->isType(ProductMapping::MAPPING_NONE)) {
                $productMapping = $mapper->findProductByAfterbuyProduct($product);
            }
            if ($productMapping->isType(ProductMapping::MAPPING_NONE)) {
                $this->log("Product {$product->getProductID()} cannot be mapped to a product/variant; skipping");
                continue;
            }
            $productsId = $productMapping->getProductsId();
            $combiId    = $productMapping->getCombiId();
            if (!$this->isValidProductsId($productsId)) {
                $this->log("Product with products_id {$productsId} does not exist, skipping.");
                continue;
            }
            //if ($this->productHasAttributes($productsId)) {
            //    $this->log("Product with products_id {$productsId} has attributes (options), skipping.");
            //    continue;
            //}
            if ($this->productHasProperties($productsId) && !empty($combiId)) {
                $this->log("Setting quantity of product variant {$productsId}x{$combiId} to {$product->getQuantity()}.");
                $this->setProductVariantsQuantity($productsId, $combiId, (float)$product->getQuantity());
            } else {
                $this->log("Setting quantity of product {$productsId} ({$product->getName()}) to {$product->getQuantity()}.");
                $this->setProductQuantity($productsId, (float)$product->getQuantity());
            }
        }
    }
    
    
    protected function setProductVariantsQuantity(int $productId, int $combiId, float $quantity)
    {
        $this->db->set('combi_quantity', $quantity)->where([
                                                               'products_id'                   => $productId,
                                                               'products_properties_combis_id' => $combiId,
                                                           ])->update('products_properties_combis');
    }
    
    
    protected function setProductQuantity(int $productsId, float $quantity)
    {
        $this->db->update('products', ['products_quantity' => $quantity], ['products_id' => $productsId]);
        set_shipping_status($productsId, null, $quantity);
    }
    
    
    protected function isValidProductsId(int $productsId): bool
    {
        $rows = $this->db->get_where('products', ['products_id' => $productsId])->num_rows();
        $this->db->reset_query();
        
        return $rows === 1;
    }
    
    
    protected function productHasAttributes(int $productsId): bool
    {
        $numAttributesRow = $this->db->select('COUNT(*) AS num_attribs')
            ->from('products_attributes')
            ->where('products_id', $productsId)
            ->get()
            ->row_array();
        $numAttributes    = $numAttributesRow['num_attribs'];
        
        return $numAttributes > 0;
    }
    
    
    protected function productHasProperties(int $productsId): bool
    {
        $numPropertiesRow = $this->db->select('COUNT(*) AS num_combis')
            ->from('products_properties_combis')
            ->where('products_id', $productsId)
            ->get()
            ->row_array();
        $numProperties    = $numPropertiesRow['num_combis'];
        
        return $numProperties > 0;
    }
    
    
    public function setLogger(LoggerInterface $logger)
    {
        $this->logger = $logger;
    }
    
    
    protected function log(string $message): void
    {
        if ($this->logger !== null) {
            $this->logger->info($message);
        }
    }
}