<?php
/* --------------------------------------------------------------
   AfterbuyOrderWriteService.php 2022-12-06
   Gambio GmbH
   http://www.gambio.de
   Copyright (c) 2022 Gambio GmbH
   Released under the GNU General Public License (Version 2)
   [http://www.gnu.org/licenses/gpl-2.0.html]
   --------------------------------------------------------------
*/

declare(strict_types=1);

use GXModules\Gambio\Afterbuy\AfterbuyCommon\Model\OrderId;
use GXModules\Gambio\Afterbuy\AfterbuyCommon\Service\AfterbuyInformationService;
use GXModules\Gambio\Afterbuy\OrderStatus\Exceptions\AfterbuyOrderStatusPaidException;
use GXModules\Gambio\Afterbuy\OrderStatus\Service\AfterbuyCheckPaidStatusService;
use Psr\Container\ContainerExceptionInterface;
use Psr\Container\NotFoundExceptionInterface;
use function Gambio\Core\Logging\logger;

/**
 * Class AfterbuyOrderWriteService
 *
 * @package GXModules\Gambio\Afterbuy\Admin\Overloads\OrderWriteService
 */
class AfterbuyOrderWriteService extends AfterbuyOrderWriteService_parent
{
    private const LOGGER_NAME = 'afterbuy';
    
    
    /**
     * Afterbuy overload of `OrderWriteService::updateOrderStatus`.
     * Send orders with status 'paid' or 'unpaid' to afterbuy if the module is installed and enabled.
     *
     * @param IdType      $orderId
     * @param IntType     $newOrderStatusId
     * @param StringType  $comment
     * @param BoolType    $customerNotified
     * @param IdType|null $customerId
     */
    public function updateOrderStatus(
        IdType     $orderId,
        IntType    $newOrderStatusId,
        StringType $comment,
        BoolType   $customerNotified,
        IdType     $customerId = null
    ) {
        parent::updateOrderStatus($orderId, $newOrderStatusId, $comment, $customerNotified, $customerId);
        
        $context  = $this->getContext($orderId, $newOrderStatusId, $comment, $customerNotified, $customerId);
        $services = $this->getServices($context);
        if (!$services) {
            return;
        }
        
        [$checkPaidStatusService, $infoService] = $services;
        if (!$infoService->isInstalledAndEnabled()) {
            return;
        }
        
        try {
            $paidStatus = $checkPaidStatusService->getPaidStatus(new OrderId($orderId->asInt()));
        } catch (AfterbuyOrderStatusPaidException $e) {
            $message = "Failed to get order paid status.\nError: {$e->getMessage()}";
            $context = $this->mergeThrowable($context, $e);
            logger(self::LOGGER_NAME)->error($message, $context);
            
            return;
        }
        
        if ($paidStatus->isPaid() || $paidStatus->isUnpaid()) {
            $afterbuyOrderSender = MainFactory::create(AfterbuyOrderSender::class, $orderId->asInt());
            try {
                $afterbuyOrderSender->processOrder();
            } catch (AfterbuyException $e) {
                $message = "Failed to send order to Afterbuy.\nError: {$e->getMessage()}";
                $context = $this->mergeThrowable($context, $e);
                logger(self::LOGGER_NAME)->error($message, $context);
            }
        }
    }
    
    
    /**
     * Provides all necessary Afterbuy-services as array (in order to extract that array to variables).
     * Returns null and logs an error if one of the services are not available in the DI-Container.
     *
     * @param array $context
     *
     * @return array{0: AfterbuyCheckPaidStatusService, 1: AfterbuyInformationService}|null
     */
    private function getServices(array $context): ?array
    {
        $container = LegacyDependencyContainer::getInstance();
        
        try {
            $checkPaidStatusService = $container->get(AfterbuyCheckPaidStatusService::class);
            $infoService            = $container->get(AfterbuyInformationService::class);
            
            return [$checkPaidStatusService, $infoService];
        } catch (NotFoundExceptionInterface|ContainerExceptionInterface $e) {
            $class   = isset($checkPaidStatusService) ? AfterbuyInformationService::class : AfterbuyCheckPaidStatusService::class;
            $message = "$class is not registered in the LegacyDependencyContainer.\nError: {$e->getMessage()}";
            $context = $this->mergeThrowable($context, $e);
            
            logger(self::LOGGER_NAME)->error($message, $context);
            
            return null;
        }
    }
    
    
    /**
     * Returns the arguments as serialized array, used for logging.
     *
     * @param IdType      $orderId
     * @param IntType     $newOrderStatusId
     * @param StringType  $comment
     * @param BoolType    $customerNotified
     * @param IdType|null $customerId
     *
     * @return array
     */
    private function getContext(
        IdType     $orderId,
        IntType    $newOrderStatusId,
        StringType $comment,
        BoolType   $customerNotified,
        IdType     $customerId = null
    ): array {
        return [
            'orderId'          => $orderId->asInt(),
            'newOrderStatusId' => $newOrderStatusId->asInt(),
            'comment'          => $comment->asString(),
            'customerNotified' => $customerNotified->asBool(),
            'customerId'       => $customerId ? $customerId->asInt() : null,
        ];
    }
    
    
    /**
     * Merges data from the Throwable into the context array and returns
     * a new copy containing error information.
     *
     * @param array     $context
     * @param Throwable $t
     *
     * @return array
     */
    private function mergeThrowable(array $context, Throwable $t): array
    {
        $context['exception'] = [
            'message' => $t->getMessage(),
            'file'    => $t->getFile(),
            'line'    => $t->getLine(),
        ];
        
        return $context;
    }
}