<?php
/* --------------------------------------------------------------
   AfterbuyRequestSender.php 2022-10-17
   Gambio GmbH
   http://www.gambio.de
   Copyright (c) 2022 Gambio GmbH
   Released under the GNU General Public License (Version 2)
   [http://www.gnu.org/licenses/gpl-2.0.html]
   --------------------------------------------------------------
*/

declare(strict_types=1);

namespace GXModules\Gambio\Afterbuy\AfterbuyCommon\App\Data\Sender;

use GuzzleHttp\Client;
use GuzzleHttp\RequestOptions;
use GXModules\Gambio\Afterbuy\AfterbuyCommon\Model\AfterbuyXmlRequest;
use GXModules\Gambio\Afterbuy\OrderExport\Exceptions\AfterbuyResponseException;

/**
 * Class AfterbuyRequestSender
 *
 * @package GXModules\Gambio\Afterbuy\OrderExport\App\Data\AfterbuySender
 */
class AfterbuyRequestSender
{
    private const HTTP_URL          = 'https://api.afterbuy.de/afterbuy/ABInterface.aspx';
    private const HTTP_CONTENT_TYPE = 'application/xml';
    
    private Client                       $client;
    private AfterbuyXmlResponseValidator $responseValidator;
    
    
    /**
     * AfterbuyRequestSender constructor.
     *
     * @param Client                       $client
     * @param AfterbuyXmlResponseValidator $responseValidator
     */
    public function __construct(Client $client, AfterbuyXmlResponseValidator $responseValidator)
    {
        $this->client            = $client;
        $this->responseValidator = $responseValidator;
    }
    
    
    /**
     * Sends the request to the Afterbuy-API.
     * This method also validates the Afterbuy-Response and throws an exception in case of an error.
     * All details are logged in the validator.
     *
     * @param AfterbuyXmlRequest $request
     *
     * @return array{status: int, headers: string[][], body: string}
     * @throws AfterbuyResponseException
     */
    public function send(AfterbuyXmlRequest $request): array
    {
        $requestOptions = [
            RequestOptions::BODY    => $request->toXmlString(),
            RequestOptions::HEADERS => [
                'Content-Type' => self::HTTP_CONTENT_TYPE,
            ],
        ];
        $response       = $this->client->post(self::HTTP_URL, $requestOptions);
        
        $responseBody = $response->getBody()->getContents();
        $this->responseValidator->validate($responseBody);
        
        return [
            'status'  => $response->getStatusCode(),
            'headers' => $response->getHeaders(),
            'body'    => $responseBody,
        ];
    }
}