<?php
/* --------------------------------------------------------------
   AfterbuyOrderPaymentInfoReader.php 2022-10-15
   Gambio GmbH
   http://www.gambio.de
   Copyright (c) 2022 Gambio GmbH
   Released under the GNU General Public License (Version 2)
   [http://www.gnu.org/licenses/gpl-2.0.html]
   --------------------------------------------------------------
*/

declare(strict_types=1);

namespace GXModules\Gambio\Afterbuy\OrderExport\App\Data\AfterbuyOrder;

use Gambio\Core\Configuration\Compatibility\ConfigurationStorageRepository;
use Gambio\Core\Configuration\Compatibility\ConfigurationStorageRepositoryBuilder;
use Gambio\Core\Configuration\Services\ConfigurationFinder;
use GXModules\Gambio\Afterbuy\OrderExport\Exceptions\AfterbuyOrderTotalDataNotFoundException;

/**
 * Class AfterbuyOrderPaymentInfoReader
 *
 * @package GXModules\Gambio\Afterbuy\OrderExport\App\Data\AfterbuyOrder
 */
class AfterbuyOrderPaymentInfoReader
{
    private const STORAGE_AFTERBUY_NAMESPACE = 'modules/gambio/afterbuy';
    private const CONFIG_ALIAS_SCAFFOLD      = 'configuration/MODULE_PAYMENT_%s_ALIAS';
    
    private ConfigurationFinder            $configurationFinder;
    private ConfigurationStorageRepository $storage;
    private AfterbuyOrderTotalReader       $orderTotalReader;
    
    
    /**
     * AfterbuyOrderPaymentInfoReader constructor.
     *
     * @param ConfigurationFinder                   $configurationFinder
     * @param ConfigurationStorageRepositoryBuilder $storageBuilder
     * @param AfterbuyOrderTotalReader              $orderTotalReader
     */
    public function __construct(
        ConfigurationFinder                   $configurationFinder,
        ConfigurationStorageRepositoryBuilder $storageBuilder,
        AfterbuyOrderTotalReader              $orderTotalReader
    ) {
        $this->configurationFinder = $configurationFinder;
        $this->storage             = $storageBuilder->build(self::STORAGE_AFTERBUY_NAMESPACE);
        $this->orderTotalReader    = $orderTotalReader;
    }
    
    
    /**
     * Tries to return the payment method of the given payment class.
     * Returns $paymentClass if nothing was found.
     *
     * @param string $paymentClass
     *
     * @return string
     */
    public function getPaymentMethod(string $paymentClass): string
    {
        $key    = sprintf(self::CONFIG_ALIAS_SCAFFOLD, strtoupper($paymentClass));
        $config = $this->configurationFinder->get($key);
        if ($config === null) {
            return $paymentClass;
        }
        
        return trim($config);
    }
    
    
    /**
     * Checks if the order was already paid. If so, the order total value is returned as float.
     * If not, the method returns null.
     *
     * @param array $data
     *
     * @return float|null
     */
    public function getAlreadyPaid(array $data): ?float
    {
        $orderStatus              = $data['orders_status'];
        $alreadyPaidStatuses      = $this->storage->get('order_status_paid');
        $alreadyPaidStatusesArray = explode(',', $alreadyPaidStatuses);
        
        // determines that all order status mark orders as paid
        if (in_array('-1', $alreadyPaidStatusesArray, true) && count($alreadyPaidStatusesArray) === 1) {
            return $this->getOrderTotalPaid($data);
        }
        
        if (in_array($orderStatus, $alreadyPaidStatusesArray, true)) {
            return $this->getOrderTotalPaid($data);
        }
        
        return null;
    }
    
    
    /**
     * Fetches the order total value from the `orders_total` table for the order.
     * The order is identified by $data's element "orders_id".
     *
     * @param array $data
     *
     * @return float|null
     */
    private function getOrderTotalPaid(array $data): ?float
    {
        $orderId = (int)$data['orders_id'];
        
        try {
            $orderTotalData = $this->orderTotalReader->fetchOrderTotalsByOrderId($orderId);
            if (array_key_exists('ot_total', $orderTotalData)) {
                return (float)$orderTotalData['ot_total'];
            }
        } catch (AfterbuyOrderTotalDataNotFoundException $e) {
        }
        
        return null;
    }
}