<?php
/* --------------------------------------------------------------
   ShippingInfo.php 2022-09-28
   Gambio GmbH
   http://www.gambio.de
   Copyright (c) 2022 Gambio GmbH
   Released under the GNU General Public License (Version 2)
   [http://www.gnu.org/licenses/gpl-2.0.html]
   --------------------------------------------------------------
*/

declare(strict_types=1);

namespace GXModules\Gambio\Afterbuy\OrderExport\Model\Request;

use DateTimeImmutable;
use GXModules\Gambio\Afterbuy\AfterbuyCommon\Model\XmlSerializable;

/**
 * Class ShippingInfo
 *
 * @package GXModules\Gambio\Afterbuy\OrderExport\Model
 */
class ShippingInfo implements XmlSerializable
{
    use RequestPriceUtilityTrait;
    
    private const DATE_FORMAT = 'd.m.Y H:i:s';
    
    private string            $shippingMethod;
    private string            $shippingGroup;
    private DateTimeImmutable $deliveryDate;
    private ?float            $shippingCost;
    private ?bool             $sendShippingMail;
    private ?string           $shippingReturnMethod;
    private ?float            $eBayShippingCost;
    
    
    public function __construct(
        string            $shippingMethod,
        string            $shippingGroup,
        DateTimeImmutable $deliveryDate,
        float             $shippingCost = null,
        bool              $sendShippingMail = null,
        string            $shippingReturnMethod = null,
        float             $eBayShippingCost = null
    ) {
        $this->shippingMethod       = $shippingMethod;
        $this->shippingGroup        = $shippingGroup;
        $this->deliveryDate         = $deliveryDate;
        $this->shippingCost         = $shippingCost;
        $this->sendShippingMail     = $sendShippingMail;
        $this->shippingReturnMethod = $shippingReturnMethod;
        $this->eBayShippingCost     = $eBayShippingCost;
    }
    
    
    /**
     * @return string
     */
    public function toXmlString(): string
    {
        $indent    = $this->indent();
        $date      = $this->deliveryDate->format(self::DATE_FORMAT);
        $optionals = $this->optionalsAsXml();
        
        if (empty($optionals)) {
            return <<<XML
$indent<ShippingInfo>
$indent    <ShippingMethod>$this->shippingMethod</ShippingMethod>
$indent    <ShippingGroup>$this->shippingGroup</ShippingGroup>
$indent    <DeliveryDate>$date</DeliveryDate>
$indent</ShippingInfo>
XML;
        }
        
        return <<<XML
$indent<ShippingInfo>
$indent    <ShippingMethod>$this->shippingMethod</ShippingMethod>
$indent    <ShippingGroup>$this->shippingGroup</ShippingGroup>
$indent    <DeliveryDate>$date</DeliveryDate>
$optionals
$indent</ShippingInfo>
XML;
    }
    
    
    /**
     * Creates a xml string of the optional values, if they are available.
     *
     * @return string
     */
    private function optionalsAsXml(): string
    {
        $indent    = str_repeat(' ', strlen($this->indent()) + 4);
        $optionals = '';
        
        if ($this->shippingCost !== null) {
            $cost      = $this->convertFloatToString($this->shippingCost);
            $optionals .= "$indent<ShippingCost>$cost</ShippingCost>\n";
        }
        if ($this->eBayShippingCost !== null) {
            $cost      = $this->convertFloatToString($this->eBayShippingCost);
            $optionals .= "$indent<eBayShippingCost>$cost</eBayShippingCost>\n";
        }
        if ($this->shippingReturnMethod !== null) {
            $optionals .= "$indent<ShippingReturnMethod>$this->shippingReturnMethod</ShippingReturnMethod>\n";
        }
        if ($this->sendShippingMail !== null) {
            $value     = $this->sendShippingMail ? 'Yes' : 'No';
            $optionals .= "$indent<SendShippingMail>$value</SendShippingMail>\n";
        }
        
        return rtrim($optionals);
    }
    
    
    /**
     * @inheritDoc
     */
    public function indent(): string
    {
        return str_repeat(' ', 12);
    }
}
