<?php
/* --------------------------------------------------------------
   AfterbuyOrderTrackingRepository.php 2022-11-14
   Gambio GmbH
   http://www.gambio.de
   Copyright (c) 2022 Gambio GmbH
   Released under the GNU General Public License (Version 2)
   [http://www.gnu.org/licenses/gpl-2.0.html]
   --------------------------------------------------------------
*/

declare(strict_types=1);

namespace GXModules\Gambio\Afterbuy\OrderTracking\App\Data;

use GXModules\Gambio\Afterbuy\AfterbuyCommon\Exceptions\AfterbuyNotEnabledException;
use GXModules\Gambio\Afterbuy\AfterbuyCommon\Exceptions\AfterbuyNotInstalledException;
use GXModules\Gambio\Afterbuy\AfterbuyCommon\Model\OrderId;
use GXModules\Gambio\Afterbuy\OrderExport\Exceptions\AfterbuyResponseException;
use GXModules\Gambio\Afterbuy\OrderTracking\Exceptions\SyncTrackingLinksFailedException;
use GXModules\Gambio\Afterbuy\OrderTracking\Model\AfterbuyOrderTrackingCode;
use GXModules\Gambio\Afterbuy\OrderTracking\Model\AfterbuyOrderTrackingCodes;

/**
 * Class AfterbuyOrderTrackingRepository
 *
 * @package GXModules\Gambio\Afterbuy\OrderTracking\App\Data
 */
class AfterbuyOrderTrackingRepository
{
    public const CONFIGURATION_AB_TRACKING_SYNC = 'last_tracking_sync';
    
    private AfterbuyOrderTrackingReader $reader;
    private AfterbuyOrderTrackingWriter $writer;
    
    
    /**
     * AfterbuyOrderTrackingRepository constructor.
     *
     * @param AfterbuyOrderTrackingReader $reader
     * @param AfterbuyOrderTrackingWriter $writer
     */
    public function __construct(AfterbuyOrderTrackingReader $reader, AfterbuyOrderTrackingWriter $writer)
    {
        $this->reader = $reader;
        $this->writer = $writer;
    }
    
    
    /**
     * Returns a list of order tracking codes from the Afterbuy XML-API.
     *
     * It is required to set up Afterbuy in the shop system, otherwise either an `AfterbuyNotInstalledException`
     * or `AfterbuyNotEnabledException` exception is thrown.
     *
     * Additionally, the method might throw an `AfterbuyResponseException` if the request to the Afterbuy XML-API
     * fails for some reason.
     *
     * @param string $lastTrackingSync
     *
     * @return AfterbuyOrderTrackingCodes
     * @throws AfterbuyNotInstalledException|AfterbuyNotEnabledException
     * @throws AfterbuyResponseException
     */
    public function getTrackingCodes(string $lastTrackingSync): AfterbuyOrderTrackingCodes
    {
        return $this->reader->getTrackingCodes($lastTrackingSync);
    }
    
    
    /**
     * Returns a list of afterbuy tracking codes for the given order.
     *
     * It is required to set up Afterbuy in the shop system, otherwise either an `AfterbuyNotInstalledException`
     * or `AfterbuyNotEnabledException` exception is thrown.
     *
     * Additionally, the method might throw an `AfterbuyResponseException` if the request to the Afterbuy XML-API
     * fails for some reason.
     *
     * @param OrderId $orderId
     *
     * @return AfterbuyOrderTrackingCodes
     * @throws AfterbuyNotInstalledException|AfterbuyNotEnabledException
     * @throws AfterbuyResponseException
     */
    public function findTrackingCodesByOrderId(OrderId $orderId): ?AfterbuyOrderTrackingCodes
    {
        return $this->reader->findTrackingCodesByOrderId($orderId);
    }
    
    
    /**
     * @param AfterbuyOrderTrackingCode $trackingCode
     *
     * @throws SyncTrackingLinksFailedException
     */
    public function saveTrackingCode(AfterbuyOrderTrackingCode $trackingCode): void
    {
        if (!$this->writer->trackingLinkExists($trackingCode)) {
            $trackingLinkShippingMethods = $this->getAfterbuyTrackingSyncShippingMethods();
            $this->writer->saveOrderTrackingCode($trackingCode, $trackingLinkShippingMethods);
            $this->writer->updateOrderStatusWithTrackingConfiguration($trackingCode->orderId());
        }
    }
    
    
    /**
     * Checks if an order exists for the tracking codes order id.
     *
     * @param AfterbuyOrderTrackingCode $trackingCode
     *
     * @return bool
     */
    public function orderExists(AfterbuyOrderTrackingCode $trackingCode): bool
    {
        return $this->reader->orderExists($trackingCode);
    }
    
    
    /**
     * Returns a list of all afterbuy tracking link synchronization configurations.
     *
     * When removing the prefix `tracking_sync_shipping_methods_` from the array key, you
     * get the parcel service provider id of the shop system.
     * The result will be cached, so subsequent calls
     *
     * @return array
     */
    public function getAfterbuyTrackingSyncShippingMethods(): array
    {
        return $this->reader->getAfterbuyTrackingSyncShippingMethods();
    }
}
