<?php
/* --------------------------------------------------------------
   AfterbuyOrderTrackingWriter.php 2022-11-16
   Gambio GmbH
   http://www.gambio.de
   Copyright (c) 2022 Gambio GmbH
   Released under the GNU General Public License (Version 2)
   [http://www.gnu.org/licenses/gpl-2.0.html]
   --------------------------------------------------------------
*/

declare(strict_types=1);

namespace GXModules\Gambio\Afterbuy\OrderTracking\App\Data;

use Doctrine\DBAL\Connection;
use Doctrine\DBAL\Exception;
use Gambio\Admin\Modules\Language\Model\Exceptions\LanguageNotFoundException;
use Gambio\Admin\Modules\Language\Model\Language;
use Gambio\Admin\Modules\ParcelService\Services\Exceptions\ParcelServiceNotFoundException;
use Gambio\Admin\Modules\ParcelService\Services\ParcelServiceReadService;
use Gambio\Core\Configuration\Compatibility\ConfigurationStorageRepository;
use Gambio\Core\Configuration\Compatibility\ConfigurationStorageRepositoryBuilder;
use Gambio\Core\Language\Services\LanguageService;
use GXModules\Gambio\Afterbuy\OrderTracking\Exceptions\SyncTrackingLinksFailedException;
use GXModules\Gambio\Afterbuy\OrderTracking\Model\AfterbuyOrderTrackingCode;

/**
 * Class AfterbuyOrderTrackingWriter
 *
 * @package GXModules\Gambio\Afterbuy\OrderTracking\App\Data
 */
class AfterbuyOrderTrackingWriter
{
    private const DEFAULT_LANGUAGE_ID     = '2';
    private const CONFIGURATION_NAMESPACE = 'modules/gambio/afterbuy';
    
    private Connection                     $connection;
    private ConfigurationStorageRepository $storage;
    private ParcelServiceReadService       $parcelServiceReadService;
    private LanguageService                $languageService;
    
    
    /**
     * AfterbuyOrderTrackingWriter constructor.
     *
     * @param Connection                            $connection
     * @param ConfigurationStorageRepositoryBuilder $builder
     * @param ParcelServiceReadService              $parcelServiceReadService
     * @param LanguageService                       $languageService
     */
    public function __construct(
        Connection                            $connection,
        ConfigurationStorageRepositoryBuilder $builder,
        ParcelServiceReadService              $parcelServiceReadService,
        LanguageService                       $languageService
    ) {
        $this->connection               = $connection;
        $this->storage                  = $builder->build(self::CONFIGURATION_NAMESPACE);
        $this->parcelServiceReadService = $parcelServiceReadService;
        $this->languageService          = $languageService;
    }
    
    
    /**
     * Checks if a tracking link already exists for the order.
     *
     * @param AfterbuyOrderTrackingCode $trackingCode
     *
     * @return bool
     * @throws SyncTrackingLinksFailedException
     */
    public function trackingLinkExists(AfterbuyOrderTrackingCode $trackingCode): bool
    {
        $qb = $this->connection->createQueryBuilder();
        
        $orderIdEq      = $qb->expr()->eq('order_id', $trackingCode->orderId());
        $trackingCodeEq = $qb->expr()->eq('tracking_code', "'{$trackingCode->trackingCode()}'");
        $where          = $qb->expr()->and($orderIdEq, $trackingCodeEq);
        
        try {
            $statement = $qb->select('*')->from('orders_parcel_tracking_codes')->where($where)->execute();
        } catch (Exception $e) {
            $message = "Failed to check if tracking code '{$trackingCode->trackingCode()}' for order id '{$trackingCode->orderId()}' already exists.\nError: {$e->getMessage()}";
            throw new SyncTrackingLinksFailedException($message, $e->getCode(), $e);
        }
        $result = $statement->fetch();
        
        return $result !== false;
    }
    
    
    /**
     * Saves the tracking code in the orders_parcel_tracking_codes table.
     *
     * @param AfterbuyOrderTrackingCode $trackingCode
     * @param array                     $trackingSyncShippingMethods
     *
     * @throws SyncTrackingLinksFailedException
     */
    public function saveOrderTrackingCode(
        AfterbuyOrderTrackingCode $trackingCode,
        array                     $trackingSyncShippingMethods
    ): void {
        $configurationKey = array_search($trackingCode->shippingMethod(), $trackingSyncShippingMethods, true);
        if ($configurationKey === false) {
            return;
        }
        $parcelServiceId = str_replace('tracking_sync_shipping_methods_', '', $configurationKey);
        try {
            $parcelService = $this->parcelServiceReadService->getParcelServiceById((int)$parcelServiceId);
        } catch (ParcelServiceNotFoundException $e) {
            $message = "Parcel service with id '$parcelServiceId' not found.\nError: {$e->getMessage()}";
            throw new SyncTrackingLinksFailedException($message, $e->getCode(), $e);
        }
        
        $shopLanguage = $this->getCurrentLanguage();
        $url          = str_replace('{TRACKING_NUMBER}',
                                    $trackingCode->trackingCode(),
                                    $parcelService->url($shopLanguage->code()));
        $data         = [
            'order_id'            => $trackingCode->orderId(),
            'tracking_code'       => $trackingCode->trackingCode(),
            'parcel_service_id'   => $parcelService->id(),
            'parcel_service_name' => $parcelService->name(),
            'language_id'         => $shopLanguage->id(),
            'url'                 => $url,
            'shipment_type'       => $trackingCode->shippingMethod(),
        ];
        
        try {
            $this->connection->insert('orders_parcel_tracking_codes', $data);
        } catch (Exception $e) {
            $message = "Database error when inserting tracking code data. Code: '{$trackingCode->trackingCode()}'. Order ID: '{$trackingCode->orderId()}'.\nError: {$e->getMessage()}";
            throw new SyncTrackingLinksFailedException($message, $e->getCode(), $e);
        }
    }
    
    
    /**
     * Updates the order status to the afterbuy configuration 'order_status_tracking_sync'.
     *
     * Additionally, updates the 'orders_status_history' table. By using the raw
     * database connection, only the database records are updated, but nothing else, like events, are triggered.
     *
     * @param int $orderId
     *
     * @throws SyncTrackingLinksFailedException
     */
    public function updateOrderStatusWithTrackingConfiguration(int $orderId): void
    {
        $configuration = $this->storage->get('order_status_tracking_sync');
        if (empty($configuration) || $configuration === '-1') {
            return;
        }
        $data  = ['orders_status' => $configuration];
        $where = ['orders_id' => $orderId];
        
        try {
            $this->connection->update('orders', $data, $where);
            
            $data = [
                'orders_id'        => $orderId,
                'orders_status_id' => $configuration,
                'date_added'       => date('Y-m-d H:i:s'),
                'comments'         => 'Tracking-Code Synchronization',
            ];
            $this->connection->insert('orders_status_history', $data);
        } catch (Exception $e) {
            $message = "Database error when updating the order status to '$configuration' for the order with id '$orderId'.\nError: {$e->getMessage()}";
            throw new SyncTrackingLinksFailedException($message, $e->getCode(), $e);
        }
    }
    
    
    /**
     * Returns the current shop language.
     *
     * @return Language
     * @throws SyncTrackingLinksFailedException
     */
    private function getCurrentLanguage(): Language
    {
        $languageId = $_SESSION['languages_id'] ?? self::DEFAULT_LANGUAGE_ID;
        $languageId = (int)$languageId;
        
        try {
            return $this->languageService->getLanguageById($languageId);
        } catch (LanguageNotFoundException $e) {
            $message = "Shop language not found.\nError: {$e->getMessage()}";
            throw new SyncTrackingLinksFailedException($message, $e->getCode(), $e);
        }
    }
}